import csv
import io
import json
import os
import sys
from datetime import datetime


def get_config(path):
    if not os.path.exists(path):
        print("config file not found")
        return False

    with open(path, "r") as f:
        return json.load(f)


def get_filename(path):
    return os.path.splitext(os.path.basename(path))[0]


def get_output_path(blob_name, output_path):
    filename = get_filename(blob_name)
    op_path = os.path.join(output_path, f"{filename}.json")
    print(f"Writing to {op_path}")
    return op_path


def get_time():
    now = datetime.now()
    return now.strftime("%Y-%m-%d %X")

def get_fields(data):
    return data.splitlines()[0].decode("utf-8").split(",")



import io
import pandas
import json
import logging
from google.cloud import storage, bigquery

from sdf.utils import get_fields, get_time, get_output_path


class SDF:
    def __init__(self, config, blob, storage_client, bigquery_client):
        self.config = config
        self.input_path = config["input_path"]
        self.output_path = config["output_path"]
        self.bucket_name = config["bucket_name"]
        self.blob = blob

        self.table_name = config["bigquery_table_name"]
        self.reprocess = config.get("reprocess", False)
        self.src = "gcs"

        self.storage_client = storage_client
        self.bigquery_client = bigquery_client

    def update_storage(self):
        bucket = self.storage_client.get_bucket(self.bucket_name)
        if self.blob is None:
            logging.error(f"input_path: {self.input_path} does not exist")
            return

        # Check if blob has already been processed in the bigquery db
        present_in_recon = bool(list(
            self.bigquery_client.query(
                f"""SELECT src_dtls from `pro-spark.recon.reconciliation` where src_dtls = "{self.blob.public_url}" """
            )
        ))
        if not self.reprocess and present_in_recon:
            print("File already processed. Skipping...")
            return

        file_contents = self.blob.download_as_string()

        metadata = {
            "_rt": self.received_timestamp,
            "_src": self.src,
            "_o": "",
            "src_dtls": self.blob.public_url,
        }

        self.processed_data = SDF.process(file_contents, metadata)

        # Create destination blob
        dest_blob = bucket.blob(get_output_path(self.blob.name, self.output_path))
        dest_blob.upload_from_string(
            SDF.custom_json_dump(self.processed_data), content_type="application/json"
        )
        return True

    def update_table(self):
        """Update table with data"""
        data = [
            {
                "src": self.src,
                "src_dtls": self.blob.public_url,
                "record_count": len(self.processed_data),
                "received_timestamp": self.received_timestamp,
                "processed_timestamp": get_time(),
            }
        ]
        print("Inserting data into recon table")
        self.bigquery_client.insert_rows_json(self.table_name, data)

    @staticmethod
    def process(data, metadata):
        f = io.StringIO(data.decode("utf-8"))
        fields = get_fields(data)
        parsed_data = pandas.read_csv(f)

        updated_data = list()
        for idx, row in parsed_data.iterrows():
            updated_data.append({"_m": metadata, "_p": {"data": dict(row)}})
        return updated_data

    @staticmethod
    def custom_json_dump(processed_data):
        """
        Takes in processed data and returns a custom JSON format with
        each line representing valid JSON.
        """
        return "\n".join(list(map(json.dumps,  processed_data)))

    def run(self):
        res = self.update_storage()
        if res:
            self.update_table()

    @property
    def received_timestamp(self):
        return self.blob.time_created.strftime("%Y-%m-%d %X")



import os
import sys
import json

from google.cloud import storage, bigquery

from sdf.utils import get_config
from sdf.sdf import SDF

def main():
    # Check if env variable is set
    if not os.environ.get("GOOGLE_APPLICATION_CREDENTIALS"):
        print("Please set 'GOOGLE_APPLICATION_CREDENTIALS' environment variable")
        os._exit(-1)

    if len(sys.argv) < 2:
        print("Provide path to config.json as argument.")
        os._exit(-1)

    config = get_config(sys.argv[1])
    storage_client = storage.Client()
    bigquery_client = bigquery.Client()

    all_blobs = list(storage_client.list_blobs(
        bucket_or_name=config["bucket_name"],
        prefix=config["input_path"])
    )
    print(f"Found {len(all_blobs)} blobs. Processing...")

    for index, blob in enumerate(all_blobs):
        print(f"Processing {index + 1} of {len(all_blobs)}: {blob.name}")
        sdf = SDF(config, blob, storage_client, bigquery_client)
        sdf.run()


if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="sankir-sdf",
    version="0.0.1",
    author="SanKir Technologies Pvt Ltd",
    author_email="sanjay@sankir.com",
    description="SDF Converter",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SanKirTech/sdf-converter",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        "console_scripts": [
            "sankir-sdf=sdf.__main__:main"
        ]
    },
    python_requires='>=3.6',
)